/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2009-2010 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors: Andrey Hristov <andrey@mysql.com>                           |
  |          Ulf Wendel <uwendel@mysql.com>                              |
  +----------------------------------------------------------------------+
*/

/* $Id: $ */

#ifndef MYSQLND_QC_PRIV_H
#define MYSQLND_QC_PRIV_H

#include "mysqlnd_qc_logs.h"


extern unsigned int mysqlnd_qc_plugin_id;

void mysqlnd_qc_ps_register_hooks();

enum_func_status mysqlnd_qc_receive_record(MYSQLND * conn, zend_uchar * buffer, size_t count TSRMLS_DC);
enum_func_status mysqlnd_qc_receive_replay(MYSQLND * conn, zend_uchar * buffer, size_t count TSRMLS_DC);

size_t mysqlnd_qc_send_record(MYSQLND * const conn, char * const buf, size_t count TSRMLS_DC);
size_t mysqlnd_qc_send_replay(MYSQLND * const conn, char * const buf, size_t count TSRMLS_DC);


typedef struct st_mysqlnd_qc_connection_data
{
	zend_bool			query_active_with_cache;
	char *				query_hash_key;
	size_t				query_hash_key_len;

	enum_func_status	(*original_receive)(MYSQLND * conn, zend_uchar * buffer, size_t count TSRMLS_DC);
	size_t				(*original_send)(MYSQLND * const conn, char * const buf, size_t count TSRMLS_DC);

	smart_str			*recorded_data;
	size_t				recorded_data_current_position;
	uint64_t			run_time;
	uint				ttl;
#ifdef NORM_QUERY_TRACE_LOG
	MYSQLND_QC_NORM_QUERY_TRACE_LOG_ENTRY * norm_qt_log_entry;
#endif
	zend_bool			multi_q_enabled;
} MYSQLND_QC_CONNECTION_DATA;


#define MYSQLND_QC_NET_SET_REPLAY(conn) \
	{ \
		(conn)->net->m.receive = mysqlnd_qc_receive_replay; \
		(conn)->net->m.send = mysqlnd_qc_send_replay; \
	}

#define MYSQLND_QC_NET_SET_RECORD(conn) \
	{ \
		(conn)->net->m.receive = mysqlnd_qc_receive_record; \
		(conn)->net->m.send = mysqlnd_qc_send_record; \
	}

#define MYSQLND_QC_NET_RESTORE_ORIGINAL(conn, data_pp) \
	{ \
		(conn)->net->m.receive = (*(data_pp))->original_receive; \
		(conn)->net->m.send = (*(data_pp))->original_send; \
	}

#define MYSQLND_QC_NET_SET_ORIGINAL(conn, data_pp) \
	{ \
		(*(data_pp))->original_receive = (conn)->net->m.receive; \
		(*(data_pp))->original_send = (conn)->net->m.send; \
	}


#define FREE_SMART_STR_AND_CONTAINER_AND_NULL(str, persistent) \
		if ((str)) { \
			smart_str_free_ex((str), (persistent)); \
			mnd_free((str)); \
			(str) = NULL; \
		}


#define COPY_HASH_KEY(src_key, src_len, dst_key, dst_len)  dst_key = src_key; dst_len = src_len;

#define TIMEVAL_TO_UINT64(tp) (uint64_t)(tp.tv_sec*1000000 + tp.tv_usec)

#define STATS_TIME_SET(time_now) \
	if (MYSQLND_QC_G(time_statistics) == FALSE) { \
		(time_now) = 0; \
	} else { \
		struct timeval __tp = {0}; \
		struct timezone __tz = {0}; \
		gettimeofday(&__tp, &__tz); \
		(time_now) = TIMEVAL_TO_UINT64(__tp); \
	} \

#define STATS_TIME_DIFF(run_time) \
	{ \
		uint64_t now; \
		STATS_TIME_SET(now); \
		(run_time) = now - (run_time); \
	}

#endif	/* MYSQLND_QC_PRIV_H */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
